@@ -19,9 +19,9 @@
 
 """Provides the interface to the AT-SPI Registry."""
 
-__id__        = "$Id: atspi.py 2471 2007-06-13 13:44:34Z shaeger $"
-__version__   = "$Revision: 2471 $"
-__date__      = "$Date: 2007-06-13 09:44:34 -0400 (Wed, 13 Jun 2007) $"
+__id__        = "$Id: atspi.py 2503 2007-06-26 20:25:55Z lmonsanto $"
+__version__   = "$Revision: 2503 $"
+__date__      = "$Date: 2007-06-26 16:25:55 -0400 (Tue, 26 Jun 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -404,6 +404,9 @@ class Accessible:
         Arguments:
         - registry: an instance of Registry
         """
+        registry.registerEventListener(
+            Accessible._onRoleChanged,
+            "object:property-change:accessible-role")
         registry.registerEventListener(
             Accessible._onNameChanged,
             "object:property-change:accessible-name")
@@ -429,6 +432,9 @@ class Accessible:
         Arguments:
         - registry: an instance of Registry
         """
+        registry.deregisterEventListener(
+            Accessible._onRoleChanged,
+            "object:property-change:accessible-role")
         registry.deregisterEventListener(
             Accessible._onNameChanged,
             "object:property-change:accessible-name")
@@ -447,6 +453,21 @@ class Accessible:
 
     shutdown = staticmethod(shutdown)
 
+    def _onRoleChanged(e):
+        """Core module event listener called when an object's role
+        changes.  Updates the cache accordingly.
+
+        Arguments:
+        - e: AT-SPI event from the AT-SPI registry
+        """
+
+        if Accessible._cache.has_key(e.source):
+            obj = Accessible._cache[e.source]
+            if obj.__dict__.has_key("role"):
+                del obj.__dict__["role"]
+
+    _onRoleChanged = staticmethod(_onRoleChanged)
+
     def _onNameChanged(e):
         """Core module event listener called when an object's name
         changes.  Updates the cache accordingly.
@@ -666,8 +687,8 @@ class Accessible:
             try:
                 self.accessible = acc._narrow(Accessibility.Accessible)
             except:
-                debug.printException(debug.LEVEL_SEVERE)
-                debug.println(debug.LEVEL_SEVERE,
+                debug.printException(debug.LEVEL_WARNING)
+                debug.println(debug.LEVEL_WARNING,
                               "atspi.py:Accessible.__init__" \
                               + " NOT GIVEN AN ACCESSIBLE!")
                 self.accessible = None
@@ -823,12 +844,12 @@ class Accessible:
                 self.accessible.unref()
             except:
                 pass
-            
+
             try:
                 Accessible.deleteAccessible(self._acc)
             except:
                 pass
-            
+
             self.accessible = None
             self._acc = None
             self.app = None
@@ -1230,7 +1251,7 @@ class Accessible:
             self.selection = selection
 
         return selection
-        
+
     def __get_document(self):
         """Returns an object that implements the Accessibility_Document
         interface for this object, or None if this object doesn't implement
