@@ -23,10 +23,10 @@ class.  For those wishing to override the braille generators, however,
 one can create a new instance and replace/extend the braille
 generators as they see fit."""
 
-__id__        = "$Id: braillegenerator.py 2343 2007-05-03 20:51:02Z richb $"
-__version__   = "$Revision: 2343 $"
-__date__      = "$Date: 2007-05-03 16:51:02 -0400 (Thu, 03 May 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: braillegenerator.py 2539 2007-07-13 18:32:02Z richb $"
+__version__   = "$Revision: 2539 $"
+__date__      = "$Date: 2007-07-13 14:32:02 -0400 (Fri, 13 Jul 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import math
@@ -46,6 +46,11 @@ class BrailleGenerator:
     primary entry point.  Subclasses can feel free to override/extend
     the brailleGenerators instance field as they see fit."""
 
+    SKIP_CONTEXT_ROLES = (rolenames.ROLE_MENU,
+                          rolenames.ROLE_MENU_BAR,
+                          rolenames.ROLE_PAGE_TAB_LIST,
+                          rolenames.ROLE_COMBO_BOX)
+
     def __init__(self, script):
 
         # The script that created us.  This allows us to ask the
@@ -295,8 +300,10 @@ class BrailleGenerator:
         regions = []
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
         text = self._script.appendString(text, self._getTextForValue(obj))
         text = self._script.appendString(text, self._getTextForRole(obj))
 
@@ -334,8 +341,10 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForAnimation", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
         text = self._script.appendString(text, self._getTextForRole(obj))
         text = self._script.appendString(text, obj.description, ": ")
 
@@ -377,13 +386,16 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForCheckBox", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
 
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text = self._script.appendString(text, "<x>")
-        else:
-            text = self._script.appendString(text, "< >")
+        text = self._script.appendString(
+            text,
+            settings.brailleCheckBoxIndicators[
+                obj.state.count(atspi.Accessibility.STATE_CHECKED)])
+
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         text = self._script.appendString(text, self._getTextForRole(obj))
 
@@ -407,17 +419,21 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForCheckMenuItem", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
 
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text = self._script.appendString(text, "<x>")
-        else:
-            text = self._script.appendString(text, "< >")
+        text = self._script.appendString(
+            text,
+            settings.brailleCheckBoxIndicators[
+                obj.state.count(atspi.Accessibility.STATE_CHECKED)])
+
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         if obj == orca_state.locusOfFocus:
             text = self._script.appendString(text, self._getTextForRole(obj))
-            text = self._script.appendString(text, self._getTextForAvailability(obj))
+            text = self._script.appendString(
+                text, self._getTextForAvailability(obj))
             text = self._script.appendString(text,
                                       self._getTextForAccelerator(obj),
                                       "")
@@ -467,9 +483,8 @@ class BrailleGenerator:
         if displayedText:
             regions.append(braille.Region(displayedText))
 
-        if settings.brailleVerbosityLevel == settings.VERBOSITY_LEVEL_VERBOSE:
-            regions.append(braille.Region(
-                " " + rolenames.getBrailleForRoleName(obj)))
+        regions.append(braille.Region(
+            " " + rolenames.getBrailleForRoleName(obj)))
 
         # Things may not have gone as expected above, so we'll do some
         # defensive programming to make sure we don't get an index out
@@ -587,8 +602,10 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForIcon", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         if obj.image:
             description = obj.image.imageDescription
@@ -662,12 +679,16 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForMenu", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
-        text = self._script.appendString(text, rolenames.getBrailleForRoleName(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(
+            text, rolenames.getBrailleForRoleName(obj))
 
         if obj == orca_state.locusOfFocus:
-            text = self._script.appendString(text, self._getTextForAvailability(obj))
+            text = self._script.appendString(
+                text, self._getTextForAvailability(obj))
             text = self._script.appendString(text,
                                       self._getTextForAccelerator(obj),
                                       "")
@@ -705,21 +726,27 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForMenuItem", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
 
         # OpenOffice check menu items currently have a role of "menu item"
         # rather then "check menu item", so we need to test if one of the
         # states is CHECKED. If it is, then add that in to the braille
         # display output. Note that we can't tell if this is a "check
-        # menu item" that is currently unchecked and braille that state. 
+        # menu item" that is currently unchecked and braille that state.
         # See Orca bug #433398 for more details.
         #
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text = self._script.appendString(text, "<x>")
+            text = self._script.appendString(
+                text,
+                settings.brailleCheckBoxIndicators[1])
+
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         if obj == orca_state.locusOfFocus:
-            text = self._script.appendString(text, self._getTextForAvailability(obj))
+            text = self._script.appendString(
+                text, self._getTextForAvailability(obj))
             text = self._script.appendString(text,
                                       self._getTextForAccelerator(obj),
                                       "")
@@ -744,12 +771,11 @@ class BrailleGenerator:
 
         regions = []
 
-        textRegion = braille.Text(obj, self._script.getDisplayedLabel(obj))
+        textRegion = braille.Text(obj,
+                                  self._script.getDisplayedLabel(obj),
+                                  " $l")
         regions.append(textRegion)
 
-        eol = braille.Region(" $l")
-        regions.append(eol)
-
         # We do not want the role at the end of text areas.
 
         return [regions, textRegion]
@@ -781,11 +807,14 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForPageTab", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         if obj == orca_state.locusOfFocus:
-            text = self._script.appendString(text, self._getTextForAvailability(obj))
+            text = self._script.appendString(
+                text, self._getTextForAvailability(obj))
             text = self._script.appendString(text,
                                       self._getTextForAccelerator(obj),
                                       "")
@@ -840,8 +869,10 @@ class BrailleGenerator:
         regions = []
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         # In Java, some push buttons don't have label and text.
         # In this case, we'll add to presentation the object description,
@@ -872,8 +903,16 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForRadioButton", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
+
+        text = self._script.appendString(
+            settings.brailleRadioButtonIndicators[
+                obj.state.count(atspi.Accessibility.STATE_CHECKED)],
+            text)
+
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         # In Java, some toggle buttons don't have label and text.
         # In this case, we'll add to presentation the object description,
@@ -882,11 +921,6 @@ class BrailleGenerator:
         if (not text) and (obj.description):
             text = self._script.appendString(text, obj.description)
 
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text = self._script.appendString(text, "&=y")
-        else:
-            text = self._script.appendString(text, "& y")
-
         text = self._script.appendString(text, self._getTextForRole(obj))
 
         regions = []
@@ -909,17 +943,21 @@ class BrailleGenerator:
         self._debugGenerator("_getBrailleRegionsForRadioMenuItem", obj)
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
-        text = self._script.appendString(text, self._script.getDisplayedText(obj))
 
-        if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            text = self._script.appendString(text, "&=y")
-        else:
-            text = self._script.appendString(text, "& y")
+        text = self._script.appendString(
+            settings.brailleRadioButtonIndicators[
+                obj.state.count(atspi.Accessibility.STATE_CHECKED)],
+            text)
+
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedText(obj))
 
         if obj == orca_state.locusOfFocus:
             text = self._script.appendString(text, self._getTextForRole(obj))
-            text = self._script.appendString(text, self._getTextForAvailability(obj))
+            text = self._script.appendString(
+                text, self._getTextForAvailability(obj))
             text = self._script.appendString(text,
                                       self._getTextForAccelerator(obj),
                                       "")
@@ -977,9 +1015,11 @@ class BrailleGenerator:
         regions = []
 
         text = ""
-        text = self._script.appendString(text, self._script.getDisplayedLabel(obj))
+        text = self._script.appendString(
+            text, self._script.getDisplayedLabel(obj))
         # Ignore the text on the slider.
-        #text = self._script.appendString(text, self._script.getDisplayedText(obj))
+        #text = self._script.appendString(
+        #    text, self._script.getDisplayedText(obj))
         text = self._script.appendString(text, self._getTextForValue(obj))
         text = self._script.appendString(text, self._getTextForRole(obj))
 
@@ -1046,6 +1086,40 @@ class BrailleGenerator:
 
         regions = []
 
+        # If this table cell has 2 children and one of them has a
+        # 'toggle' action and the other does not, then present this
+        # as a checkbox where:
+        # 1) we get the checked state from the cell with the 'toggle' action
+        # 2) we get the label from the other cell.
+        # See Orca bug #376015 for more details.
+        #
+        if obj.childCount == 2:
+            cellOrder = []
+            hasToggle = [ False, False ]
+            for i in range(0, obj.childCount):
+                action = obj.child(i).action
+                if action:
+                    for j in range(0, action.nActions):
+                        if action.getName(j) == "toggle":
+                            hasToggle[i] = True
+                            break
+
+            if hasToggle[0] and not hasToggle[1]:
+                cellOrder = [ 0, 1 ]
+            elif not hasToggle[0] and hasToggle[1]:
+                cellOrder = [ 1, 0 ]
+            if cellOrder:
+                for i in cellOrder:
+                    [cellRegions, focusRegion] = \
+                            self._getBrailleRegionsForTableCell(obj.child(i))
+                    if len(regions):
+                        regions.append(braille.Region(" "))
+                    else:
+                        cellFocusedRegion = focusRegion
+                    regions.append(cellRegions[0])
+                regions = [regions, cellFocusedRegion]
+                return regions
+
         # [[[TODO: WDW - Attempt to infer the cell type.  There's a
         # bunch of stuff we can do here, such as check the EXPANDABLE
         # state, check the NODE_CHILD_OF relation, etc.  Logged as
@@ -1060,6 +1134,21 @@ class BrailleGenerator:
                 if action.getName(i) == "toggle":
                     obj.role = rolenames.ROLE_CHECK_BOX
                     regions = self._getBrailleRegionsForCheckBox(obj)
+
+                    # If this table cell doesn't have any label associated 
+                    # with it then also braille the table column header.
+                    # See Orca bug #455230 for more details.
+                    #
+                    label = self._script.getDisplayedText( \
+                        self._script.getRealActiveDescendant(obj))
+                    action = obj.action
+                    if label == None or len(label) == 0:
+                        n = obj.parent.table.getColumnAtIndex(obj.index)
+                        header = obj.parent.table.getColumnHeader(n)
+                        accHeader = atspi.Accessible.makeAccessible(header)
+                        regions[0].append(braille.Region(" "))
+                        regions[0].append(braille.Region(accHeader.name))
+
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
@@ -1152,7 +1241,7 @@ class BrailleGenerator:
                             self._getBrailleRegionsForTableCell(cell)
                         if len(rowRegions):
                             rowRegions.append(braille.Region(" "))
-                        rowRegions.append(cellRegions[0])
+                        rowRegions.extend(cellRegions)
                         if i == column:
                             focusRowRegion = cellRegions[0]
                 regions = [rowRegions, focusRowRegion]
@@ -1229,7 +1318,8 @@ class BrailleGenerator:
             title = self._script.getDisplayedLabel(obj)
 
         text = title
-        text = self._script.appendString(text, rolenames.getBrailleForRoleName(obj))
+        text = self._script.appendString(
+            text, rolenames.getBrailleForRoleName(obj))
 
         regions = []
         regions.append(braille.Region(text))
@@ -1413,10 +1503,7 @@ class BrailleGenerator:
 
         regions = []
         parent = obj.parent
-        if parent \
-           and ((parent.role == rolenames.ROLE_MENU) \
-                or (parent.role == rolenames.ROLE_MENU_BAR) \
-                or (parent.role == rolenames.ROLE_PAGE_TAB_LIST)):
+        if parent and (parent.role in self.SKIP_CONTEXT_ROLES):
             parent = parent.parent
         while parent and (parent.parent != parent):
             # [[[TODO: WDW - we might want to include more things here
@@ -1465,7 +1552,10 @@ class BrailleGenerator:
         parent = obj.parent
         if parent and parent.table:
             row = parent.table.getRowAtIndex(obj.index)
-            desc = parent.table.getRowDescription(row)
+            if row >= 0:
+                desc = parent.table.getRowDescription(row)
+            else:
+                desc = None
             if desc and len(desc):
                 if settings.brailleRolenameStyle \
                        == settings.VERBOSITY_LEVEL_VERBOSE:
@@ -1481,7 +1571,10 @@ class BrailleGenerator:
                 regions.append(braille.Region(text))
 
             col = parent.table.getColumnAtIndex(obj.index)
-            desc = parent.table.getColumnDescription(col)
+            if col >= 0:
+                desc = parent.table.getColumnDescription(col)
+            else:
+                desc = None
             if desc and len(desc):
                 if settings.brailleVerbosityLevel \
                        == settings.VERBOSITY_LEVEL_VERBOSE:
