@@ -19,9 +19,9 @@
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2173 2007-03-14 15:33:38Z richb $"
-__version__   = "$Revision: 2173 $"
-__date__      = "$Date: 2007-03-14 11:33:38 -0400 (Wed, 14 Mar 2007) $"
+__id__        = "$Id: focus_tracking_presenter.py 2551 2007-07-24 17:13:10Z richb $"
+__version__   = "$Revision: 2551 $"
+__date__      = "$Date: 2007-07-24 13:13:10 -0400 (Tue, 24 Jul 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -74,7 +74,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._gidleLock      = threading.Lock()
         self.noFocusTimestamp = 0.0
 
-        orca_state.activeScript = None
+        self.setActiveScript(None)
 
         if settings.debugEventQueue:
             self._enqueueEventCount = 0
@@ -265,6 +265,68 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         return script
 
+    def setActiveScript(self, newScript):
+        """Set the new active script.
+
+        Arguments:
+        - newScript: the new script to be made active.
+        """
+
+        try:
+            # If old ("factory") settings don't exist yet, save
+            # a set, else restore the old application settings.
+            #
+            if not self._oldAppSettings:
+                self._oldAppSettings = \
+                    orca_state.activeScript.saveOldAppSettings()
+            else:
+                orca_state.activeScript.restoreOldAppSettings( \
+                    self._oldAppSettings)
+
+            orca_state.activeScript.deactivate()
+        except:
+            pass
+
+        orca_state.activeScript = newScript
+
+        try:
+            orca_state.activeScript.activate()
+        except:
+            pass
+
+    def _cleanupCache(self):
+        """Looks for defunct accessible objects in the cache and removes them.
+        """
+
+        objectsRemoved = 0
+        for obj in atspi.Accessible._cache.values():
+            try:
+                if obj.state.count(atspi.Accessibility.STATE_DEFUNCT):
+                    atspi.Accessible.deleteAccessible(obj)
+                    objectsRemoved += 1
+                else:
+                    # Try to force a COMM_FAILURE
+                    #
+                    obj.toString()
+            except CORBA.COMM_FAILURE:
+                atspi.Accessible.deleteAccessible(obj)
+                objectsRemoved += 1
+
+        debug.println(debug.LEVEL_FINEST,
+                      "_cleanupCache: %d objects removed." % objectsRemoved)
+
+    def _cleanupGarbage(self):
+        """Cleans up garbage on the heap."""
+        import gc
+        gc.collect()
+        for obj in gc.garbage:
+            try:
+                if isinstance(obj, atspi.Accessible):
+                    gc.garbage.remove(obj)
+                    obj.__del__()
+            except:
+                pass
+
     def _reclaimScripts(self):
         """Compares the list of known scripts to the list of known apps,
         deleting any scripts as necessary.
@@ -281,16 +343,27 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             apps = []
             desktop = self.registry.desktop
             for i in range(0, desktop.childCount):
-                acc = desktop.getChildAtIndex(i)
-                app = atspi.Accessible.makeAccessible(acc)
-                if app:
-                    apps.insert(0, app)
+                try:
+                    acc = desktop.getChildAtIndex(i)
+                    app = atspi.Accessible.makeAccessible(acc)
+                    if app:
+                        apps.insert(0, app)
+                except:
+                    pass
 
             for app in self._knownScripts.keys():
                 if apps.count(app) == 0:
                     script = self._knownScripts[app]
                     self._deregisterEventListeners(script)
+
+                    # Provide a bunch of hints to the garbage collector
+                    # that we just don't care about this stuff any longer.
+                    # Note the "app.app = None" - that helps remove a
+                    # cycle of the application referring to itself.
+                    #
                     del self._knownScripts[app]
+                    app.app = None
+                    del app
                     del script
         except:
             debug.printException(debug.LEVEL_FINEST)
@@ -301,13 +374,14 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
     #                                                                      #
     ########################################################################
 
-    def loadAppSettings(self, app):
+    def loadAppSettings(self, script):
         """Load the users application specific settings for an app.
 
         Arguments:
-        - app: the Python app
+        - script: the current active script.
         """
 
+        app = script.app
         settingsPackages = settings.settingsPackages
         moduleName = settings.getScriptModuleName(app)
         module = None
@@ -321,13 +395,24 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 try:
                     debug.println(debug.LEVEL_FINEST,
                                   "Looking for settings at %s.py..." % name)
-                    if self. _knownAppSettings.has_key(name):
-                        reload(self._knownAppSettings[name])
-                    else:
+                    if not self._knownAppSettings.has_key(name):
                         self._knownAppSettings[name] = \
                             __import__(name, globals(), locals(), [''])
+                    reload(self._knownAppSettings[name])
                     debug.println(debug.LEVEL_FINEST,
                                   "...found %s.py" % name)
+
+                    # Setup the user's application specific key bindings.
+                    # (if any).
+                    #
+                    if hasattr(self._knownAppSettings[name],
+                               "overrideAppKeyBindings"):
+                        script.overrideAppKeyBindings = \
+                            self._knownAppSettings[name].overrideAppKeyBindings
+                        script.keyBindings = \
+                         self._knownAppSettings[name].overrideAppKeyBindings( \
+                            script, script.keyBindings)
+
                     break
                 except ImportError:
                     debug.println(debug.LEVEL_FINEST,
@@ -387,7 +472,27 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         try:
             if event.source.role == rolenames.ROLE_TOOL_TIP:
-                return
+                # Check that it's okay to present tool tips. Always present
+                # tooltips initiated by the user pressing Control-F1 on the
+                # keyboard.
+                #
+                if isinstance(orca_state.lastInputEvent, \
+                              input_event.KeyboardEvent):
+                    if not orca_state.lastNonModifierKeyEvent.event_string == "F1":
+                        return
+
+                    # Mouse move events don't update orca_state.lastInputEvent
+                    # so it's possible the user accidentally nudged the
+                    # mouse and generated another tooltip event. If the
+                    # current time minus the last keyboard event time is
+                    # greater than 0.2 seconds, than just ignore this tooltip
+                    # event.
+                    #
+                    currentTime =  time.time()
+                    if (currentTime - orca_state.lastInputEvent.time) > 0.2:
+                        return
+                elif not settings.presentToolTips:
+                    return
         except:
             pass
 
@@ -400,11 +505,9 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if event.source == atspi.Accessible.makeAccessible(
                                    self.registry.desktop):
                 self._reclaimScripts()
-                #import gc
-                #gc.collect()
-                #print "In process, garbage:", gc.garbage
-                #for obj in gc.garbage:
-                #    print "   referrers:", obj, gc.get_referrers(obj)
+                self._cleanupCache()
+                if settings.debugMemoryUsage:
+                    self._cleanupGarbage()
             return
 
         try:
@@ -421,7 +524,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if (not debug.eventDebugFilter) \
                 or (debug.eventDebugFilter \
                     and debug.eventDebugFilter.match(event.type)):
-                debug.printDetails(debug.LEVEL_FINEST, "    ", event.source)
+                if not event.type.startswith("mouse:"):
+                    debug.printDetails(debug.LEVEL_FINEST, "    ", event.source)
 
         except CORBA.COMM_FAILURE:
             debug.printException(debug.LEVEL_WARNING)
@@ -456,7 +560,17 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         retryCount = 0
         oldLocusOfFocus = orca_state.locusOfFocus
         try:
-            s = self._getScript(event.source.app)
+            # If we've received a mouse event, then don't try to get
+            # event.source.app because the top most parent is of role 
+            # unknown, which will cause an ERROR message to be displayed.
+            # See Orca bug #409731 for more details.
+            #
+            if not event.type.startswith("mouse:"):
+                s = self._getScript(event.source.app)
+            else:
+                s = orca_state.activeScript
+            if not s:
+                return
         except:
             s = None
             debug.printException(debug.LEVEL_WARNING)
@@ -483,25 +597,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
-                        orca_state.activeScript = \
-                            self._getScript(event.source.app)
                         debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
                                       + orca_state.activeScript.name)
 
-                        # If old ("factory") settings don't exist yet, save
-                        # a set, else restore the old application settings.
-                        #
-                        if not self._oldAppSettings:
-                            self._oldAppSettings = \
-                                  orca_state.activeScript.saveOldAppSettings()
-                        else:
-                            orca_state.activeScript.restoreOldAppSettings( \
-                                                         self._oldAppSettings)
+                        self.setActiveScript(self._getScript(event.source.app))
 
                         # Load in the application specific settings for the
                         # app for this event (if found).
                         #
-                        appSettings = self.loadAppSettings(event.source.app)
+                        appSettings = self.loadAppSettings( \
+                                                      orca_state.activeScript)
 
                         # Tell BrlTTY which commands we care about.
                         #
@@ -567,16 +672,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         event = None
         if isinstance(e, input_event.KeyboardEvent):
             if e.type == atspi.Accessibility.KEY_PRESSED_EVENT:
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYPRESS '%s' (%d)"
                               % (e.event_string, e.hw_code))
             elif e.type == atspi.Accessibility.KEY_RELEASED_EVENT:
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYRELEASE '%s' (%d)"
                               % (e.event_string, e.hw_code))
             event = e
         elif isinstance(e, input_event.BrailleEvent):
-            debug.println(debug.LEVEL_FINEST,
+            debug.println(debug.LEVEL_ALL,
                           "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
             event = e
         else:
@@ -626,7 +731,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if (not debug.eventDebugFilter) \
                 or (debug.eventDebugFilter \
                     and debug.eventDebugFilter.match(e.type)):
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "---------> QUEUEING EVENT %s" % e.type)
 
         if event:
@@ -637,10 +742,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           ...acquired")
-            if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           calling queue.put...")
-            if settings.debugEventQueue:
                 debug.println(debug.LEVEL_ALL,
                               "           (full=%s)" % self._eventQueue.full())
             self._eventQueue.put(event)
@@ -691,12 +794,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
             if isinstance(event, input_event.KeyboardEvent):
                 if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
-                    debug.println(debug.LEVEL_FINEST,
+                    debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYPRESS '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
                     pressRelease = "PRESS"
                 elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT:
-                    debug.println(debug.LEVEL_FINEST,
+                    debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYRELEASE '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
                     pressRelease = "RELEASE"
@@ -708,7 +811,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                               "\n^^^^^ PROCESS KEY %s EVENT %s ^^^^^"\
                               % (pressRelease, event.event_string))
             elif isinstance(event, input_event.BrailleEvent):
-                debug.println(debug.LEVEL_FINEST,
+                debug.println(debug.LEVEL_ALL,
                               "DEQUEUED BRAILLE COMMAND %d <----------" \
                               % event.event)
                 debug.println(debug.eventDebugLevel,
@@ -722,7 +825,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 if (not debug.eventDebugFilter) \
                     or (debug.eventDebugFilter \
                         and debug.eventDebugFilter.match(event.type)):
-                    debug.println(debug.LEVEL_FINEST,
+                    debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED EVENT %s <----------" \
                                   % event.type)
                     debug.println(debug.eventDebugLevel,
@@ -751,6 +854,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 if noFocus:
                     if settings.gilSleepTime:
                         time.sleep(settings.gilSleepTime)
+                    # Translators: this is intended to be a short phrase to
+                    # speak and braille to tell the user that no component
+                    # has keyboard focus.
+                    #
                     message = _("No focus")
                     if settings.brailleVerbosityLevel == \
                         settings.VERBOSITY_LEVEL_VERBOSE:
@@ -843,6 +950,23 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         if orca_state.activeScript:
             orca_state.activeScript.visualAppearanceChanged(event, obj)
 
+    def _saveAppStates(self):
+        """Saves script and application state information."""
+        self._appStateInfo = []
+        for script in self._knownScripts.values():
+            self._appStateInfo.append([script.app, script.getAppState()])
+
+    def _restoreAppStates(self):
+        """Restores script and application state information."""
+        try:
+            for [app, appState] in self._appStateInfo:
+                script = self._getScript(app)
+                script.setAppState(appState)
+        except:
+            pass
+
+        self._appStateInfo = None
+
     def activate(self):
         """Called when this presentation manager is activated."""
 
@@ -852,7 +976,9 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._oldAppSettings = None
         self._defaultScript  = None
 
-        orca_state.activeScript = self._getScript(None)
+        self._restoreAppStates()
+
+        self.setActiveScript(self._getScript(None))
 
         # Tell BrlTTY which commands we care about.
         #
@@ -878,6 +1004,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
     def deactivate(self):
         """Called when this presentation manager is deactivated."""
 
+        self._saveAppStates()
+
         for eventType in self._listenerCounts.keys():
             self.registry.deregisterEventListener(self._enqueueEvent,
                                                   eventType)
@@ -887,4 +1015,4 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._oldAppSettings = None
         self._defaultScript  = None
 
-        orca_state.activeScript = None
+        self.setActiveScript(None)
