@@ -20,10 +20,10 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 1979 2007-02-04 16:44:37Z wwalker $"
-__version__   = "$Revision: 1979 $"
-__date__      = "$Date: 2007-02-04 11:44:37 -0500 (Sun, 04 Feb 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: mag.py 2507 2007-06-27 19:01:05Z wwalker $"
+__version__   = "$Revision: 2507 $"
+__date__      = "$Date: 2007-06-27 15:01:05 -0400 (Wed, 27 Jun 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import bonobo
@@ -253,6 +253,11 @@ def __dumpPropertyBag(obj):
     slots = pbag.getKeys("")
     print "  Available slots: ", pbag.getKeys("")
     for slot in slots:
+        # These crash the magnifier since it doesn't know how to marshall
+        # them to us.
+        #
+        if slot in ["cursor-set", "smoothing-type"]:
+            continue
         print "    About '%s':" % slot
         print "    Doc Title:", pbag.getDocTitle(slot)
         print "    Type:", pbag.getType(slot)
@@ -427,6 +432,8 @@ def applySettings():
     _maxROIX = _sourceDisplayBounds.x2 - (_roiWidth / 2)
     _maxROIY = _sourceDisplayBounds.y2 - (_roiHeight / 2)
 
+    _magnifier.addZoomRegion(_zoomer)
+
     #print "MAGNIFIER PROPERTIES:", _magnifier
     #__dumpPropertyBag(_magnifier)
     #print "ZOOMER PROPERTIES:", _zoomer
@@ -575,8 +582,16 @@ def shutdown():
 
     atspi.Registry().deregisterEventListener(__onMouseEvent,"mouse:abs")
 
-    _magnifier.clearAllZoomRegions()
-    _magnifier.dispose()
+    # Someone might have killed the magnifier on us.  They shouldn't
+    # have done so, but we need to be able to recover if they did.
+    # See http://bugzilla.gnome.org/show_bug.cgi?id=375396.
+    #
+    try:
+        _magnifier.clearAllZoomRegions()
+        _magnifier.dispose()
+    except:
+        debug.printException(debug.LEVEL_WARNING)
+
     _magnifier = None
 
     _initialized = False
