@@ -21,9 +21,9 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 2366 2007-05-09 21:22:51Z richb $"
-__version__   = "$Revision: 2366 $"
-__date__      = "$Date: 2007-05-09 17:22:51 -0400 (Wed, 09 May 2007) $"
+__id__        = "$Id: settings.py 2521 2007-07-06 20:46:49Z mpeders $"
+__version__   = "$Revision: 2521 $"
+__date__      = "$Date: 2007-07-06 16:46:49 -0400 (Fri, 06 Jul 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -103,6 +103,7 @@ userCustomizableSettings = [
     "enableBrailleGrouping",
     "brailleVerbosityLevel",
     "brailleRolenameStyle",
+    "brailleSelectorIndicator",
     "enableBrailleMonitor",
     "enableMagnifier",
     "enableMagCursor",
@@ -129,6 +130,11 @@ userCustomizableSettings = [
     "sayAllStyle",
     "keyboardLayout",
     "speakBlankLines",
+    "enabledSpokenTextAttributes",
+    "enabledBrailledTextAttributes",
+    "textAttributesBrailleIndicator",
+    "enableProgressBarUpdates",
+    "progressBarUpdateInterval"
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -188,6 +194,16 @@ BRAILLE_ROLENAME_STYLE_SHORT = 0 # three letter abbreviations
 BRAILLE_ROLENAME_STYLE_LONG  = 1 # full rolename
 brailleRolenameStyle    = BRAILLE_ROLENAME_STYLE_LONG
 
+# Braille Selection Indicator (see brailleSelectorIndicator).
+# The values represent the character to be used in the attrOr
+# field of brlAPI's writeStruct.
+#
+BRAILLE_SEL_NONE = '\x00' # 00000000
+BRAILLE_SEL_7    = '\x40' # 01000000
+BRAILLE_SEL_8    = '\x80' # 10000000
+BRAILLE_SEL_BOTH = '\xc0' # 11000000
+brailleSelectorIndicator = BRAILLE_SEL_BOTH
+
 # Speech punctuation levels (see verbalizePunctuationStyle).
 #
 PUNCTUATION_STYLE_NONE = 3
@@ -219,6 +235,11 @@ speechPitchDelta        = 0.5
 #
 httpServerPort          = 20433
 
+# The number of attempts to retry setting up an HTTP server
+# connection (each time incrementing the port number by 1).
+#
+maxHttpServerRetries   = 20
+
 # If True, enable speech.
 #
 enableSpeech            = True
@@ -241,7 +262,7 @@ HYPERLINK_VOICE         = "hyperlink"
 
 voices = {
     DEFAULT_VOICE   : ACSS({}),
-    UPPERCASE_VOICE : ACSS({ACSS.AVERAGE_PITCH : 6}),
+    UPPERCASE_VOICE : ACSS({ACSS.AVERAGE_PITCH : 5.6}),
     HYPERLINK_VOICE : ACSS({})
 }
 
@@ -263,6 +284,11 @@ enableBrailleGrouping   = False
 #
 enableBrailleMonitor    = False
 
+# Strings used to indicate checkbox/radio button states in braille:
+#
+brailleCheckBoxIndicators = ["< >", "<x>"]
+brailleRadioButtonIndicators = ["& y", "&=y"]
+
 # If True, enable magnification.
 #
 enableMagnifier                  = False
@@ -392,6 +418,19 @@ speakBlankLines         = True
 #
 readTableCellRow    = True
 
+# If True, enable speaking of progress bar updates.
+#
+enableProgressBarUpdates = False
+
+# The interval (in seconds) between speaking progress bar updates. A value
+# of zero means that progress bar updates should not be spoken at all.
+#
+progressBarUpdateInterval = 10
+
+# The complete list of possible text attributes.
+#
+allTextAttributes = "bg-color:; bg-full-height:; bg-stipple:; direction:; editable:; family-name:; fg-color:; fg-stipple:; font-effect:none; indent:0; invisible:; justification:left; language:; left-margin:; line-height:100%; paragraph-style:Default; pixels-above-lines:; pixels-below-lines:; pixels-inside-wrap:; right-margin:; rise:; scale:; size:; stretch:; strikethrough:false; style:normal; text-decoration:none; text-rotation:0; text-shadow:none; underline:none; variant:; vertical-align:baseline; weight:400; wrap-mode:; writing-mode:lr-tb;"
+
 # The default set of text attributes to speak to the user. Specific
 # application scripts (or individual users can override these values if
 # so desired. Each of these text attributes is of the form <key>:<value>;
@@ -400,7 +439,27 @@ readTableCellRow    = True
 # it won't be spoken. If no value part is given, then that attribute will
 # always be spoken.
 
-enabledTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+enabledSpokenTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+
+# The default set of text attributes to be brailled for the user. Specific
+# application scripts (or individual users can override these values if
+# so desired. Each of these text attributes is of the form <key>:<value>;
+# The <value> part will be the "default" value for that attribute. In
+# other words, if the attribute for a given piece of text has that value,
+# it won't be spoken. If no value part is given, then that attribute will
+# always be brailled.
+
+enabledBrailledTextAttributes = "size:; family-name:; weight:400; indent:0; underline:none; strikethrough:false; justification:left; style:normal;"
+
+# Text Attributes Braille Indicator (see textAttributesBrailleIndicator).
+# The values represent the character to be used in the attrOr
+# field of brlAPI's writeStruct.
+#
+TEXT_ATTR_BRAILLE_NONE = '\x00' # 00000000
+TEXT_ATTR_BRAILLE_7    = '\x40' # 01000000
+TEXT_ATTR_BRAILLE_8    = '\x80' # 10000000
+TEXT_ATTR_BRAILLE_BOTH = '\xc0' # 11000000
+textAttributesBrailleIndicator = TEXT_ATTR_BRAILLE_NONE
 
 # The limit to enable a repeat character count to be spoken.
 # If set to 0, then there will be no repeat character count.
@@ -474,6 +533,12 @@ useBonoboMain           = True
 #
 debugEventQueue         = False
 
+# If True, we collect information regarding memory usage and provide
+# keystrokes to dump the usage information to the console:
+# Orca+Ctrl+F8 for brief, Orca+Shift+Ctrl+F8 for detailed.
+#
+debugMemoryUsage        = False
+
 # The timeout value (in seconds) and callback used to determine if
 # Orca has hung or not.  The only setting one should muck with here is
 # the timeoutTime unless you want to create a custom callback handler
@@ -650,6 +715,6 @@ setScriptMapping(re.compile(_('bug-buddy')), "gnome_segv2")
 setScriptMapping(re.compile(_('vte')), "gnome-terminal")
 
 # Translators: see the regular expression note above.  This is for
-# the time-admin application.
+# supporting pidgin, which is the new name for gaim.
 #
-setScriptMapping(re.compile(_('time-admin')), "users-admin")
+setScriptMapping(re.compile(_('pidgin')), "gaim")
